package eu.beesoft.gantt.demo;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import eu.beesoft.gaia.util.AbstractBean;
import eu.beesoft.gaia.util.Objects;
import eu.beesoft.gantt.Dependency;
import eu.beesoft.gantt.TimelineObject;

public class Task extends AbstractBean implements TimelineObject {

	private String taskId;

	private String name;

	private String description;

	private Date startDate;

	private Date endDate;

	private long effort;

	private boolean milestone;

	private Task supertask;

	private List<Task> subtask;

	private List<Dependency> dependencies;

	private Resource assignee;



	public Task () {
		// empty
	}



	public String getName () {
		return name;
	}



	public void setName (String name) {
		String old = this.name;
		this.name = name;
		firePropertyChange ("name", old, name);
	}



	/**
	 * @return the description
	 */
	public String getDescription () {
		return description;
	}



	/**
	 * @param description
	 *            the description to set
	 */
	public void setDescription (String description) {
		this.description = description;
	}



	public String getTaskId () {
		return taskId;
	}



	public void setTaskId (String taskId) {
		String old = this.taskId;
		this.taskId = taskId;
		firePropertyChange ("taskId", old, taskId);

	}



	public Date getStartDate () {
		if ((startDate == null) && isSummary ()) {
			updateStart ();
		}
		return startDate;
	}



	public void setStartDate (Date start) {
		if (Objects.equals (this.startDate, start)) {
			return;
		}
		Date old = this.startDate;
		this.startDate = start;
		if (supertask != null) {
			supertask.updateStart ();
		}
		firePropertyChange ("startDate", old, start);
	}



	private void updateStart () {
		if (isSummary ()) {
			Date thisStart = null;
			List<Task> tasks = getSubtask ();
			for (Task task : tasks) {
				Date taskStart = task.getStartDate ();
				if (taskStart != null) {
					if ((thisStart == null) || thisStart.after (taskStart)) {
						thisStart = taskStart;
					}
				}
			}
			if (thisStart != null) {
				setStartDate (thisStart);
			}
		}
	}



	public Date getEndDate () {
		if ((endDate == null) && isSummary ()) {
			updateEnd ();
		}
		return endDate;
	}



	public void setEndDate (Date end) {
		if (Objects.equals (this.endDate, end)) {
			return;
		}
		Date old = this.endDate;
		this.endDate = end;
		if (supertask != null) {
			supertask.updateEnd ();
		}
		firePropertyChange ("endDate", old, end);
	}



	private void updateEnd () {
		if (isSummary ()) {
			Date thisEnd = null;
			List<Task> tasks = getSubtask ();
			for (Task task : tasks) {
				Date taskEnd = task.getEndDate ();
				if (taskEnd != null) {
					if ((thisEnd == null) || thisEnd.before (taskEnd)) {
						thisEnd = taskEnd;
					}
				}
			}
			if (thisEnd != null) {
				setEndDate (thisEnd);
			}
		}
	}



	public long getEffort () {
		if ((effort == 0) && isSummary ()) {
			updateEffort ();
		}
		return effort;
	}



	public void setEffort (long effort) {
		if (this.effort == effort) {
			return;
		}
		long old = this.effort;
		this.effort = effort;
		if (supertask != null) {
			supertask.updateEffort ();
		}
		firePropertyChange ("effort", old, effort);
	}



	private void updateEffort () {
		if (isSummary ()) {
			long thisEffort = 0;
			List<Task> tasks = getSubtask ();
			for (Task task : tasks) {
				thisEffort += task.getEffort ();
			}
			setEffort (thisEffort);
		}
	}



	private void updateAll () {
		updateStart ();
		updateEnd ();
		updateEffort ();
	}



	public boolean isMilestone () {
		return milestone;
	}



	public void setMilestone (boolean milestone) {
		if (this.milestone == milestone) {
			return;
		}
		boolean old = this.milestone;
		this.milestone = milestone;
		firePropertyChange ("milestone", old, milestone);
	}



	public Task getSupertask () {
		return supertask;
	}



	private void setSupertask (Task supertask) {
		Task old = this.supertask;
		if (old != supertask) {
			if (old != null) {
				this.supertask = null;
				old.removeSubtask (this);
			}
			this.supertask = supertask;
			firePropertyChange ("supertask", old, supertask);
		}
	}



	public Project getProject () {
		Task parent = getSupertask ();
		if (parent != null) {
			return parent.getProject ();
		}
		return null;
	}



	public boolean isSummary () {
		return (getSubtask ().size () > 0);
	}



	public List<Task> getSubtask () {
		if (subtask == null) {
			return Collections.emptyList ();
		}
		return subtask;
	}



	public void setSubtask (List<Task> subtask) {
		if ((this.subtask != null) && !this.subtask.isEmpty ()) {
			List<Task> tmp = new ArrayList<Task> (subtask);
			for (Task task : tmp) {
				removeSubtask (task);
			}
		}
		for (Task task : subtask) {
			addSubtask (task);
		}
	}



	public void addSubtask (Task task) {
		if (subtask == null) {
			subtask = new ArrayList<Task> ();
		}
		else if (subtask.contains (task)) {
			return;
		}
		subtask.add (task);
		task.setSupertask (this);
		updateAll ();
		firePropertyChange ("subtask", null, subtask);
	}



	public void removeSubtask (Task task) {
		if (subtask != null) {
			if (subtask.remove (task)) {
				if (task.getSupertask () == this) {
					task.setSupertask (null);
				}
				updateAll ();
				firePropertyChange ("subtask", null, subtask);
			}
		}
	}



	public void shiftSubtask (Task task, int index) {
		subtask.remove (task);
		subtask.add (index, task);
		firePropertyChange ("subtask", null, subtask);
	}



	public List<Dependency> getDependencies () {
		if (dependencies == null) {
			return Collections.emptyList ();
		}
		return dependencies;
	}



	public void setDependencies (List<Dependency> dependencies) {
		if ((this.dependencies != null) && !this.dependencies.isEmpty ()) {
			List<Dependency> tmp = new ArrayList<Dependency> (dependencies);
			for (Dependency dependency : tmp) {
				removeDependency (dependency);
			}
		}
		for (Dependency dependency : dependencies) {
			addDependency (dependency);
		}
	}



	public void addDependency (Dependency dependency) {
		if (dependencies == null) {
			dependencies = new ArrayList<Dependency> ();
		}
		else if (dependencies.contains (dependency)) {
			return;
		}
		dependencies.add (dependency);
		firePropertyChange ("dependencies", null, dependencies);
	}



	public void removeDependency (Dependency dependency) {
		if (dependencies != null) {
			if (dependencies.remove (dependency)) {
				firePropertyChange ("dependencies", null, dependencies);
			}
		}
	}



	/**
	 * @return the assignee
	 */
	public Resource getAssignee () {
		return assignee;
	}



	/**
	 * @param assignee
	 *            the assignee to set
	 */
	public void setAssignee (Resource assignee) {
		if (Objects.equals (this.assignee, assignee)) {
			return;
		}
		Resource old = this.assignee;
		if (old != null) {
			this.assignee = null;
			old.removeTask (this);
		}
		this.assignee = assignee;
		if (assignee != null) {
			assignee.addTask (this);
		}
		firePropertyChange ("assignee", old, assignee);
	}



	@Override
	public String toString () {
		return getName ();
	}
}
